@@ -31,7 +31,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Config.py 463 2011-02-09 21:57:52Z st-cea $
+# $Id: Config.py 470 2011-02-24 20:27:28Z st-cea $
 
 """
 CLI configuration classes
@@ -62,7 +62,7 @@ class ClushConfig(ConfigParser.ConfigParser, object):
                       "connect_timeout" : "30",
                       "command_timeout" : "0",
                       "history_size" : "100",
-                      "color" : WHENCOLOR_CHOICES[0],
+                      "color" : WHENCOLOR_CHOICES[-1], # auto
                       "verbosity" : "%d" % VERB_STD,
                       "node_count" : "yes",
                       "fd_max" : "16384" }
