@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright CEA/DAM/DIF (2010)
+# Copyright CEA/DAM/DIF (2010, 2011)
 #  Contributor: Stephane THIELL <stephane.thiell@cea.fr>
 #
 # This file is part of the ClusterShell library.
@@ -31,7 +31,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Display.py 465 2011-02-12 21:45:41Z st-cea $
+# $Id: Display.py 482 2011-03-09 00:13:50Z st-cea $
 
 """
 CLI results display class
@@ -52,6 +52,7 @@ class Display(object):
     """
     Output display class for command line scripts.
     """
+    COLOR_RESULT_FMT = "\033[32m%s\033[0m"
     COLOR_STDOUT_FMT = "\033[34m%s\033[0m"
     COLOR_STDERR_FMT = "\033[31m%s\033[0m"
     SEP = "-" * 15
@@ -118,12 +119,25 @@ class Display(object):
             self._display = self._print_buffer
     line_mode = property(_getlmode, _setlmode)
 
-    def _format_header(self, nodeset):
-        """Format nodeset-based header."""
+    def _format_nodeset(self, nodeset):
+        """Sub-routine to format nodeset string."""
         if self.regroup:
             return nodeset.regroup(self.groupsource, noprefix=self.noprefix)
         return str(nodeset)
 
+    def format_header(self, nodeset, indent=0):
+        """Format nodeset-based header."""
+        indstr = " " * indent
+        nodecntstr = ""
+        if self.node_count and len(nodeset) > 1:
+            nodecntstr = " (%d)" % len(nodeset)
+        if not self.label:
+            return ""
+        return self.color_stdout_fmt % ("%s%s\n%s%s%s\n%s%s" % \
+            (indstr, self.SEP,
+             indstr, self._format_nodeset(nodeset), nodecntstr,
+             indstr, self.SEP))
+
     def print_line(self, nodeset, line):
         """Display a line with optional label."""
         if self.label:
@@ -149,24 +163,31 @@ class Display(object):
 
     def _print_buffer(self, nodeset, content):
         """Display a dshbak-like header block and content."""
-        nodecntstr = ""
-        if self.node_count and len(nodeset) > 1:
-            nodecntstr = " (%d)" % len(nodeset)
-        header = self.color_stdout_fmt % ("%s\n%s%s\n%s\n" % (self.SEP,
-                                            self._format_header(nodeset),
-                                            nodecntstr, self.SEP))
-        self.out.write("%s%s\n" % (header, content))
-        
+        self.out.write("%s\n%s\n" % (self.format_header(nodeset), content))
+
     def _print_lines(self, nodeset, msg):
         """Display a MsgTree buffer by line with prefixed header."""
+        out = self.out
         if self.label:
-            header = self.color_stdout_fmt % \
-                        ("%s: " % self._format_header(nodeset))
-            for line in msg:
-                self.out.write("%s%s\n" % (header, line))
+            if self.gather:
+                header = self.color_stdout_fmt % \
+                            ("%s: " % self._format_nodeset(nodeset))
+                for line in msg:
+                    out.write("%s%s\n" % (header, line))
+            else:
+                for node in nodeset:
+                    header = self.color_stdout_fmt % \
+                                ("%s: " % self._format_nodeset(node))
+                    for line in msg:
+                        out.write("%s%s\n" % (header, line))
         else:
-            for line in msg:
-                self.out.write(line + '\n')
+            if self.gather:
+                for line in msg:
+                    out.write(line + '\n')
+            else:
+                for node in nodeset:
+                    for line in msg:
+                        out.write(line + '\n')
 
     def vprint(self, level, message):
         """Utility method to print a message if verbose level is high
