@@ -31,12 +31,13 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Error.py 460 2011-02-07 23:22:45Z st-cea $
+# $Id: Error.py 470 2011-02-24 20:27:28Z st-cea $
 
 """
 CLI error handling helper functions
 """
 
+import os.path
 import signal
 import sys
 
@@ -59,7 +60,7 @@ GENERIC_ERRORS = (EngineNotSupportedError,
                   IOError,
                   KeyboardInterrupt)
 
-def handle_generic_error(excobj, prog=sys.argv[0]):
+def handle_generic_error(excobj, prog=os.path.basename(sys.argv[0])):
     """handle error given `excobj' generic script exception"""
     try:
         raise excobj
