@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Poll.py 460 2011-02-07 23:22:45Z st-cea $
+# $Id: Poll.py 500 2011-05-29 16:18:05Z st-cea $
 
 """
 A poll() based ClusterShell Engine.
@@ -39,6 +39,7 @@ The poll() system call is available on Linux and BSD.
 """
 
 import errno
+import os
 import select
 import sys
 import time
@@ -125,7 +126,7 @@ class EnginePoll(Engine):
                 elif timeo == -1:
                     timeo = timeout
 
-                self.reg_clifds_changed = False
+                self._current_loopcnt += 1
                 evlist = self.polling.poll(timeo * 1000.0 + 1.0)
 
             except select.error, (ex_errno, ex_strerror):
@@ -142,25 +143,21 @@ class EnginePoll(Engine):
                 if event & select.POLLNVAL:
                     raise EngineException("Caught POLLNVAL on fd %d" % fd)
 
-                if self.reg_clifds_changed:
-                    self._debug("REG CLIENTS CHANGED - Aborting current evlist")
-                    # Oops, reconsider evlist by calling poll() again.
-                    break
-
                 # get client instance
                 client, fdev = self._fd2client(fd)
-                if not client or fdev is None:
+                if client is None:
                     continue
 
                 # process this client
-                client._processing = True
+                client._current_client = client
 
                 # check for poll error condition of some sort
                 if event & select.POLLERR:
                     self._debug("POLLERR %s" % client)
                     self.unregister_writer(client)
-                    client.file_writer.close()
-                    client.file_writer = None
+                    os.close(client.fd_writer)
+                    client.fd_writer = None
+                    client._current_client = None
                     continue
 
                 # check for data to read
@@ -177,16 +174,15 @@ class EnginePoll(Engine):
                         self._debug("EngineClientEOF %s" % client)
                         if fdev & Engine.E_READ:
                             self.remove(client)
+                        client._current_client = None
                         continue
 
                 # or check for end of stream (do not handle both at the same
                 # time because handle_read() may perform a partial read)
                 elif event & select.POLLHUP:
                     self._debug("POLLHUP fd=%d %s (r%s,e%s,w%s)" % (fd,
-                        client.__class__.__name__, client.reader_fileno(),
-                        client.error_fileno(), client.writer_fileno()))		
-                    client._processing = False
-
+                        client.__class__.__name__, client.fd_reader,
+                        client.fd_error, client.fd_writer))		
                     if fdev & Engine.E_READ:
                         if client._events & Engine.E_ERROR:
                             self.modify(client, 0, fdev)
@@ -197,20 +193,18 @@ class EnginePoll(Engine):
                             self.modify(client, 0, fdev)
                         else:
                             self.remove(client)
-                    continue
 
                 # check for writing
                 if event & select.POLLOUT:
                     self._debug("POLLOUT fd=%d %s (r%s,e%s,w%s)" % (fd,
-                        client.__class__.__name__, client.reader_fileno(),
-                        client.error_fileno(), client.writer_fileno()))
+                        client.__class__.__name__, client.fd_reader,
+                        client.fd_error, client.fd_writer))
                     assert fdev == Engine.E_WRITE
                     assert client._events & fdev
                     self.modify(client, 0, fdev)
                     client._handle_write()
 
-                # post processing
-                client._processing = False
+                client._current_client = None
 
                 # apply any changes occured during processing
                 if client.registered:
