@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: NodeSet.py 485 2011-03-12 10:09:26Z st-cea $
+# $Id: NodeSet.py 512 2011-06-09 16:58:25Z st-cea $
 
 """
 Cluster node set module.
@@ -209,11 +209,7 @@ class RangeSet:
         provided list.
         """
         inst = RangeSet(autostep=autostep)
-        for rng in rnglist:
-            if isinstance(rng, RangeSet):
-                inst.update(rng)
-            else:
-                inst.update(RangeSet(rng))
+        inst.updaten(rnglist)
         return inst
 
     @classmethod
@@ -238,6 +234,11 @@ class RangeSet:
         odict = self.__dict__.copy()
         # pickle includes current serial version
         odict['_version'] = RangeSet._VERSION
+        # workaround for pickling object from Python < 2.5
+        if sys.version_info < (2, 5, 0):
+            # Python 2.4 can't pickle slice objects
+            odict['_ranges'] = [((sli.start, sli.stop, sli.step), pad) \
+                                    for sli, pad in self._ranges]
         return odict
 
     def __setstate__(self, dic):
@@ -247,6 +248,10 @@ class RangeSet:
         if getattr(self, '_version', 0) < RangeSet._VERSION:
             self._ranges = [(slice(start, stop + 1, step), pad) \
                                 for start, stop, step, pad in self._ranges]
+        elif self._ranges and type(self._ranges[0][0]) is not slice:
+            # workaround for object pickled from Python < 2.5
+            self._ranges = [(slice(start, stop, step), pad) \
+                                for (start, stop, step), pad in self._ranges]
 
     def __len__(self):
         """
@@ -447,7 +452,7 @@ class RangeSet:
         Return the element at index or a subrange when a slice is specified.
         """
         if isinstance(index, slice):
-            inst = RangeSet(autostep=self._autostep)
+            inst = RangeSet(autostep=self._autostep + 1)
             sl_start, sl_stop, sl_step = \
                 RangeSet._extractslice(index, lambda o: o._length, self)
             sl_next = sl_start
@@ -595,7 +600,7 @@ class RangeSet:
                 for j in xrange(start, stop_adjust, step):
                     self._ranges.append((slice(j, j + 1, step), pad))
             self._length = (stop_adjust - start - 1) / step + 1
-        elif step > 1:
+        elif step > 1 or self._autostep < 1E100:
             # use generic expand/fold method in that case
             self._add_range_exfold(start, stop, step, pad)
         else:
@@ -713,6 +718,21 @@ class RangeSet:
         for sli, pad in rangeset._ranges:
             self.add_range(sli.start, sli.stop, sli.step, pad)
 
+    def updaten(self, rangesets):
+        """
+        Update a rangeset with the union of itself and several others.
+        """
+        # Use expand/compute/fold for performance in that specific case.
+        pad = 0
+        items, rgpad = self._expand()
+        for rng in rangesets:
+            if not isinstance(rng, RangeSet):
+                rng = RangeSet(rng)
+            for sli, pad in rng._ranges:
+                items += xrange(sli.start, sli.stop, sli.step)
+        items.sort()
+        self._ranges, self._length = self._fold(items, pad or rgpad)
+
     def clear(self):
         """
         Remove all ranges from this rangeset.
@@ -1166,6 +1186,31 @@ class NodeSetBase(object):
             # create new pattern with no rangeset (single node)
             self._patterns[pat] = None
 
+    def _addn(self, pat, rangesets):
+        """
+        Add nodes from a (pat, list of rangesets).
+        """
+        assert len(rangesets) > 0
+        # get patterns dict entry
+        pat_e = self._patterns.get(pat)
+        
+        # check for single node presence
+        single = True
+        for rng in rangesets:
+            if rng is not None:
+                single = False
+                break
+
+        if pat_e is None:
+            if single:
+                self._patterns[pat] = None
+            else:
+                pat_e = self._patterns[pat] = rangesets[0].copy()
+                pat_e.updaten(rangesets[1:])
+        else:
+            assert not single
+            pat_e.updaten(rangesets)
+
     def union(self, other):
         """
         s.union(t) returns a new set with elements from both s and t.
@@ -1198,6 +1243,21 @@ class NodeSetBase(object):
         for pat, rangeset in other._patterns.iteritems():
             self._add(pat, rangeset)
 
+    def updaten(self, others):
+        """
+        s.updaten(t) returns nodeset s with elements added from given list.
+        """
+        # optimized for pattern homogeneous clusters (a common case)
+        patd = {}
+        # gather rangesets from each common nodeset pattern
+        for other in others:
+            self._binary_sanity_check(other)
+            for pat, rangeset in other._patterns.iteritems():
+                patd.setdefault(pat, []).append(rangeset)
+        # for each pattern, add all needed rangesets in once
+        for pat, rgsets in patd.iteritems():
+            self._addn(pat, rgsets)
+        
     def clear(self):
         """
         Remove all nodes from this nodeset.
@@ -1655,8 +1715,7 @@ class NodeSet(NodeSetBase):
         provided list.
         """
         inst = NodeSet(autostep=autostep, resolver=resolver)
-        for node in nodelist:
-            inst.update(node)
+        inst.updaten(nodelist)
         return inst
 
     @classmethod
@@ -1853,6 +1912,13 @@ class NodeSet(NodeSetBase):
         nodeset = self._parser.parse(other, self._autostep)
         NodeSetBase.update(self, nodeset)
 
+    def updaten(self, others):
+        """
+        s.updaten(list) returns nodeset s with elements added from given list.
+        """
+        NodeSetBase.updaten(self, \
+            [self._parser.parse(other, self._autostep) for other in others])
+
     def intersection_update(self, other):
         """
         s.intersection_update(t) returns nodeset s keeping only
