@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Ssh.py 459 2011-02-07 23:13:55Z st-cea $
+# $Id: Ssh.py 502 2011-05-29 20:06:43Z st-cea $
 
 """
 ClusterShell Ssh/Scp support
@@ -40,7 +40,6 @@ This module implements OpenSSH engine client and task's worker.
 
 import copy
 import os
-import signal
 
 from ClusterShell.NodeSet import NodeSet
 from ClusterShell.Worker.EngineClient import EngineClient
@@ -95,10 +94,6 @@ class Ssh(EngineClient):
             task.info("print_debug")(task, "SSH: %s" % ' '.join(cmd_l))
 
         self.popen = self._exec_nonblock(cmd_l)
-        self.file_error = self.popen.stderr
-        self.file_reader = self.popen.stdout
-        self.file_writer = self.popen.stdin
-
         self.worker._on_start()
         return self
 
@@ -116,16 +111,19 @@ class Ssh(EngineClient):
             prc = self.popen.poll()
             if prc is None:
                 # process is still running, kill it
-                # NOTE: later, use Popen.send_signal(SIGKILL) [python2.6+]
-                os.kill(self.popen.pid, signal.SIGKILL)
+                self.popen.kill()
         prc = self.popen.wait()
         if prc >= 0:
             rc = prc
 
-        self.popen.stdin.close()
-        self.popen.stdout.close()
-        if self.popen.stderr:
-            self.popen.stderr.close()
+        os.close(self.fd_reader)
+        self.fd_reader = None
+        if self.fd_error:
+            os.close(self.fd_error)
+            self.fd_error = None
+        if self.fd_writer:
+            os.close(self.fd_writer)
+            self.fd_writer = None
 
         if rc >= 0:
             self.worker._on_node_rc(self.key, rc)
@@ -218,11 +216,6 @@ class Scp(Ssh):
         if self.preserve:
             cmd_l.append("-p")
 
-        user = task.info("scp_user") or task.info("ssh_user")
-        if user:
-            cmd_l.append("-l")
-            cmd_l.append(user)
-
         connect_timeout = task.info("connect_timeout", 0)
         if connect_timeout > 0:
             cmd_l.append("-oConnectTimeout=%d" % connect_timeout)
@@ -236,8 +229,8 @@ class Scp(Ssh):
             if ssh_options:
                 cmd_l += ssh_options.split()
 
+        user = task.info("scp_user") or task.info("ssh_user")
         if self.reverse:
-            user = task.info("ssh_user")
             if user:
                 cmd_l.append("%s@%s:%s" % (user, self.key, self.source))
             else:
@@ -247,8 +240,6 @@ class Scp(Ssh):
                          (os.path.basename(self.source), self.key)))
         else:
             cmd_l.append(self.source)
-
-            user = task.info("ssh_user")
             if user:
                 cmd_l.append("%s@%s:%s" % (user, self.key, self.dest))
             else:
@@ -258,10 +249,6 @@ class Scp(Ssh):
             task.info("print_debug")(task, "SCP: %s" % ' '.join(cmd_l))
 
         self.popen = self._exec_nonblock(cmd_l)
-        self.file_reader = self.popen.stdout
-        self.file_error = self.popen.stderr
-        self.file_writer = self.popen.stdin
-
         self.worker._on_start()
         return self
 
