@@ -38,7 +38,7 @@ class CLIClushConfigTest(unittest.TestCase):
         options, _ = parser.parse_args([])
         config = ClushConfig(options, filename=f.name)
         self.assert_(config != None)
-        self.assertEqual(config.color, WHENCOLOR_CHOICES[0])
+        self.assertEqual(config.color, WHENCOLOR_CHOICES[-1])
         self.assertEqual(config.verbosity, VERB_STD)
         self.assertEqual(config.fanout, 64)
         self.assertEqual(config.node_count, True)
@@ -63,7 +63,7 @@ class CLIClushConfigTest(unittest.TestCase):
         options, _ = parser.parse_args([])
         config = ClushConfig(options, filename=f.name)
         self.assert_(config != None)
-        self.assertEqual(config.color, WHENCOLOR_CHOICES[0])
+        self.assertEqual(config.color, WHENCOLOR_CHOICES[-1])
         self.assertEqual(config.verbosity, VERB_STD)
         self.assertEqual(config.node_count, True)
         self.assertEqual(config.fanout, 64)
