@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 # ClusterShell.NodeSet test suite
 # Written by S. Thiell 2007-12-05
-# $Id: NodeSetTest.py 485 2011-03-12 10:09:26Z st-cea $
+# $Id: NodeSetTest.py 512 2011-06-09 16:58:25Z st-cea $
 
 
 """Unit test for NodeSet"""
 
 import copy
+import pickle
 import sys
 import unittest
 
@@ -1237,6 +1238,17 @@ class NodeSetTest(unittest.TestCase):
         self.assertEqual(str(nodeset3), \
             "glycine[68,4780-4999],zclu[115-118,130,166-170]")
 
+    def testPickle(self):
+        """test pickling of NodeSet"""
+        dump = pickle.dumps(NodeSet("foo[1-100]"))
+        self.assertNotEqual(dump, None)
+        nodeset = pickle.loads(dump)
+        self.assertEqual(nodeset, NodeSet("foo[1-100]"))
+        self.assertEqual(str(nodeset), "foo[1-100]")
+        self.assertEqual(nodeset[0], "foo1")
+        self.assertEqual(nodeset[1], "foo2")
+        self.assertEqual(nodeset[-1], "foo100")
+
 
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(NodeSetTest)
