@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 # ClusterShell.NodeSet.RangeSet test suite
 # Written by S. Thiell
-# $Id: RangeSetTest.py 485 2011-03-12 10:09:26Z st-cea $
+# $Id: RangeSetTest.py 512 2011-06-09 16:58:25Z st-cea $
 
 
 """Unit test for RangeSet"""
 
 import copy
+import pickle
 import sys
 import unittest
 
@@ -690,6 +691,17 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(str(r1), "115-117,130,167-170,4780-4999")
         self.assertEqual(str(r2), "115-118,130,166-170,4780-4999")
 
+    def testPickle(self):
+        """test pickling of RangeSet"""
+        dump = pickle.dumps(RangeSet("1-100"))
+        self.assertNotEqual(dump, None)
+        rngset = pickle.loads(dump)
+        self.assertEqual(rngset, RangeSet("1-100"))
+        self.assertEqual(str(rngset), "1-100")
+        self.assertEqual(rngset[0], 1)
+        self.assertEqual(rngset[1], 2)
+        self.assertEqual(rngset[-1], 100)
+
 
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(RangeSetTest)
